# RTLS Platform Services

## Common

### Description

This GitHub repository is a mono repository for building a backend services
like `Platform REST API Server`, `Platform Message Processor` and `Solution Platform Sync`.
The repository is organized into three main components:

- `rtls-platform-rest-server`: The web server component that is implementing platform REST API for managing main
  entities like `nodes`, `tags`, `trackables` etc.
- `rtls-platform-message-processor`: The message processor component is a heart of the server and responsible for
  processing incoming messages from `rtls-platform-bin-json-convertor`, and `rtls-gateway`. The main responsibilities
  of message processor is extending raw system messages with platform level information and messages proxying.
- `rtls-platform-sync`: Is the module that is synchronized data between solutions and platform rest servers and
  databases, starts on `rtls-server` restart

### REST API Documentation

Swagger documentation is accessible by route `{host}/api/platform/docs/`

### Technologies

- `rtls-platform-rest-server`: NodeJs, Express, Redis
- `rtls-platform-message-processor`: Nodejs
- `rtls-platform-sync`: Redis, MariaDb

### Requirements

- Ubuntu 22.04
- NodeJs v20

### Project local setup

- clone git repository

- install `NodeJs` via `nvm`:

```shell
curl -o- https://raw.githubusercontent.com/nvm-sh/nvm/v0.39.3/install.sh | bash
nvm install 20.5.1

```

- or install regular `NodeJs`

```shell
curl -fsSL https://deb.nodesource.com/setup_20.x | sudo bash -
apt-get install -y nodejs

```

- install yarn

```shell
npm i -g yarn

```

- install `MariaDB`

```shell
sudo apt update
sudo apt install mariadb-server

```

- install `Redis`

```shell
sudo apt update
sudo apt install redis-server

```

- install server node packages `yarn`
- run platform rest server `yarn rs:start`, by default server will run on port 3001
- run platform message processor `yarn mp:start`, by default listen ZMQ 7710 port
- sync `yarn sync:start`, execute synchronization between solution storage and platform storage

### Configure

For packages configuration create `.env` files:

```shell
cp apps/message-processor/.env.defaults apps/message-processor/.env
cp apps/rest-server/.env.defaults apps/rest-server/.env
cp apps/sync/.env.defaults apps/sync/.env

```

and update environment variables (e.g. database password)

### Version increment

Scripts for version increment

- message-processor

```shell
yarn version:patch:mp

```

- rest-server

```shell
yarn version:patch:rs

```

- sync

```shell
yarn version:patch:sync

```

### Build packages

For build package require to push branch

- message-processor `master_mp`
- rest-server `master_rs`
- sync `master_sync`

## Conventions

### Code style

#### Formatting rules

- please, follow the eslint rules described in project (with this purpose setting up your IDE with
  the [.eslintrc](.eslintrc));

#### Naming rules

- please name the variables and constants in a human-readable manner (ex. use `getUsersSql` instead of `sql1`);
- please use the `camelCase` style in name the variables and constants (ex. `getUsersSql` instead of `get_users_sql`);
- please use upper case naming for constants that are not changeable in project (ex. `MAX_ROWS_NUMBER`);
- please use `JavaScript` conventions in naming the classes, constructors(functions) and others entities (classes should
  be named wih the first capital latter);

### Tests

- run tests coverage with the command `yarn test:coverage` in application
  directory (`apps/message-processor`, `apps/rest-server`)

#### Unit tests

- unit tests should be places at the same level as the tested file
- unit test name should include the name of the file that is contains covered functionality and obey a
  pattern `{filename}.test.mjs`
- run tests with the command `yarn test:unit` in application directory (`apps/message-processor`, `apps/rest-server`)
- run tests coverage with the command `yarn test:unit:coverage` in application
  directory (`apps/message-processor`, `apps/rest-server`)

#### Integration tests

- e2e tests should be places at the `e2e/integration` folder of each application
- e2e test name should include the name of the entity that is testing and obey a pattern `{filename}.test.mjs`
- run tests with the command `yarn test:integration` in application
  directory (`apps/message-processor`, `apps/rest-server`)
- run tests coverage with the command `yarn test:integration:coverage` in application
  directory (`apps/message-processor`, `apps/rest-server`)
