import zmq from 'zeromq/v5-compat.js';
import express from 'express';
import controller from './controller/index.mjs';
import logger from '../logger.mjs';
import { APP_ZMQ_LISTEN_CHANNELS } from '../../system/constants.mjs';

let server = null;
let service = null;
const app = express();
const CONNECT_ATTEMPTS = 10;
let connectionAttempt = 0;

app.use('/', (req, res) => {
    res.status(200).send({});
});

/**
 * @param params
 * @property params.serverPort - message processor service port
 * @property params.pubSubUrl - url for ZMQ subscription
 */
export function startServer({ serverPort, pubSubUrl }) {
    // Http Server is using with the tests purposes
    server = app.listen(serverPort, () => {
        logger.info('Server listening on port:', serverPort);
        startService({ pubSubUrl });
    });
}

export async function stopServer() {
    if (!server) return;

    server.close();
    logger.info('Server stopped');
}

/**
 *
 * @param service ZMQ sub socket
 */
function subscribeService(service) {
    APP_ZMQ_LISTEN_CHANNELS.forEach(channel => {
        service.subscribe(channel);
    })
}

/**
 *
 * @param params
 * @property params.pubSubUrl - url for ZMQ subscription
 */
function startService({ pubSubUrl }) {
    try {
        service = zmq.socket('sub');
        service.connect(pubSubUrl);
        subscribeService(service);
        logger.info('ZMQ Service listen to: ', pubSubUrl);
        logger.info('ZMQ Service subscribed on: ', JSON.stringify(APP_ZMQ_LISTEN_CHANNELS));
        service.on('message', controller);

        service.on('offline', () => {
            logger.warn('ZMQ Client Offline');
        });

        service.on('error', (error) => {
            logger.error('ZMQ Service Error', error);
        });
    } catch (err) {
        logger.error('ZMQ Service try connect to: ', pubSubUrl);
        logger.error('ZMQ Service connection', err);
        if (connectionAttempt <= CONNECT_ATTEMPTS) {
            connectionAttempt += 1;
            setTimeout(function () {
                startService({ pubSubUrl });
            }, 5000 * connectionAttempt);
        } else {
            logger.info(`Stop trying connect to ZMQ after ${connectionAttempt} attempts`);
        }
    }
}
