import zmq from 'zeromq/v5-compat.js';
import { CoreNotificator } from '../core/index.mjs';

export class ZmqNotificator extends CoreNotificator {
    #transport = null;

    /**
     * @constructor
     * @param config
     * @property config.url - ZMQ connection string
     */
    constructor(config) {
        super();

        const { url } = config;
        this.#transport = zmq.socket('push');
        this.#transport.connect(url);

        if (!this.#transport) throw new Error('Transport not found');
    }

    /**
     * @method
     * @param {Object} message - message object with notification details
     * @property {String} message.channel - notification channel
     * @property {String} message.object - notification message object name
     * @property {String} message.action - notification message action name
     * @property {Object} message.data - notification message data
     * @return {String}
     */
    formatMessage({ channel, data }) {
        // const ts = new Date().toISOString();
        return `${channel} ${JSON.stringify(data)}`;
    }

    /**
     * @method
     * @param {Object} message - message object with notification details
     * @property {String} message.channel - notification channel
     * @property {String} message.object - notification message object name
     * @property {String} message.action - notification message action name
     * @property {Object} message.data - notification message data
     * @return {Promise<void>}
     */
    async notify(message) {
        return this.#sendMessage(this.formatMessage(message));
    }

    /**
     * @method
     * @param {String} message - notification message
     * @return {Promise<void>}
     */
    #sendMessage = async (message) => {
        await this.#transport.send([message]);
    };
}
