import ModelBase from '../ModelBase.mjs';

export class Device extends ModelBase {
    deviceResource = 'devices';
    nodeIdLimit;

    constructor() {
        super();
    }

    /**
     * @method
     * @param {String} mac  - hardware device MAC address
     * @param {String} ts  - timestamp in ISO 8601 format
     * @returns {Promise<null>}
     */
    async updateLastHeard(mac, ts) {
        const nodeUid = await this.repository.node.findUidByMac(mac);
        if (!nodeUid) return null;

        const data = {
            uid: nodeUid,
            mac,
            last_heard: ts,
        };
        await this.repository.node.save(data);
        return null;
    }
    /** @method
     * @param {String} mac  - hardware device MAC address
     * @returns {Promise<Object>}
     */
    async findByMac(mac) {
        const node = await this.repository.node.findByMac(mac);
        if (!node) return null;
        return node;
    }

    /** @method check device if exist and create if does not
     * @param {String} mac  - hardware device MAC address
     * @param {String} ts  - timestamp in ISO 8601 format
     * @param {String} uid  - unique id
     * @returns {Promise<Object>}
     */
    async findByMacOrCreate(mac, ts, uid) {
        return await this.repository.node.findByMacOrCreate(
            mac,
            ts,
            uid,
           async () => await this.repository.nodeId.nextValue()
        );
    }
}
