/**
 * @function
 * @param {Object} rawPos - input position from ZMQ
 * @param {String} trackableUid - trackable unique identifier
 * @param {String} sublocationUid - sublocation unique identifier
 * @param {String} type - position type
 * @return {Object} - extended position
 */
export function formatPosition(rawPos, trackableUid, sublocationUid, type) {
    const slvrMode = (rawPos?.data?.slvr_mode || '').toUpperCase();
    const er = slvrMode === '0D' ? rawPos.data.fom * 128 : null; // for 1D firmware always send fom equal to 50
    return {
        tk_uid: trackableUid,
        ts: rawPos.ts,
        data: {
            x: rawPos.data.x,
            y: rawPos.data.y,
            z: rawPos.data.z,
            sl_uid: sublocationUid,
            type,
            mac: rawPos.mac.toUpperCase(),
            er,
            slvr_mode: slvrMode,
            fom: rawPos.data.fom,
            inactive: rawPos.data.inactive,
        },
    };
}
