function listToMap(list, itemField) {
    return list.reduce((memo, item) => {
        memo[item[itemField]] = { ...item };
        return memo;
    }, {});
}

/**
 * @param {NetworkSettingCreateBody} params  - Network Settings Create Request Body
 * @param {String} params.created_at  - timestamp ISO 8601
 * @param {String} params.updated_at  - timestamp ISO 8601
 * @returns {Object}
 */
export function prepareSettingToSave(params) {
    return {
        location_uid: params.location_uid,
        created_at: params.created_at,
        lsb: params.lsb,
        updated_at: params.updated_at,
        value: {
            ...params.value,
            masters: params.value.masters && params.value.masters.length ? listToMap(params.value.masters, 'anchor_uid') : null,
            distributors: params.value.distributors && params.value.distributors.length ? listToMap(params.value.distributors, 'anchor_uid'): null,
        },
    };
}

/**
 * @param {Object} body  - Repository Network Settings
 * @returns {Object} - Network Settings Response
 */
export function formatSettingResponse(body) {
    const masters = body.value && body.value.masters ? Object.values(body.value.masters) : [];
    const distributors = body.value && body.value.distributors ? Object.values(body.value.distributors) : [];
    return {
        ...body,
        value: {
            active_schedule_uid: body.value.active_schedule_uid,
            lsb: body.value.lsb,
            masters: masters.map((item) => ({
                ...item,
                sfd: Number(item.sfd),
                auto_sync: !!Number(item.auto_sync),
            })),
            distributors: distributors.map((item) => ({
                anchor_uid: item.anchor_uid,
                master_uid: item.master_uid || null,
                slot: Number(item.slot),
            })),
            tdoa_version: 3,
        },
    };
}
