import UseCaseBase from '../UseCaseBase.mjs';
import { NetworkLsbSchedule, ValidationError } from '../../models/index.mjs';

export class NetworkLsbScheduleUpdate extends UseCaseBase {
    static validationRules = {
        uid: ['strict_string', 'not_empty', 'required'],
        name: ['strict_string', 'not_empty'],
        content: [{
            list_of_objects: [{
                source: ['strict_string', 'not_empty', 'required'],
                destination: ['strict_string', 'not_empty', 'required'],
                slot: ['strict_integer', { min_number: 0 }, { max_number: 254 }, 'required'],
                section: ['strict_integer', { one_of: [1, 0] }, 'required'],
                dimension: ['strict_string', { one_of: ['0d', '1d', '2d', '0D', '1D', '2D'] }, 'not_empty', 'required', 'to_upper_case'],
                boundary: ['strict_bool', 'required'],
                margin: ['strict_integer', { multiple: 250 }, { min_number: 0 }, { max_number: 50000 }, 'required']
            }]
        }]
    };

    async execute(params) {
        if (params.content) {
            if (params.content.length === 0) {
                throw new ValidationError('{"content":"CANNOT_BE_EMPTY"}');
            }
            for (const content of params.content) {
                if (content.dimension === '2D' && content.section === 1) {
                    throw new ValidationError('For content.dimension 2D available only content.section 0');
                }
            }
        }
        const schedule = await new NetworkLsbSchedule().update(params);

        await this.notificator.notify(NetworkLsbSchedule.updatedMessage(schedule));

        return schedule;
    }
}
