import UseCaseBase from '../UseCaseBase.mjs';
import { NetworkSetting } from '../../models/index.mjs';

export class NetworkSettingSave extends UseCaseBase {
    static validationRules = {
        location_uid: ['strict_string', 'not_empty', 'required'],
        value: [{
            nested_object: {
                active_schedule_uid: ['strict_string', 'not_empty', 'required'],
                lsb: ['not_empty', 'required', { one_of: ['on', 'off', 'ON', 'OFF'] }, 'to_upper_case'],
                masters: ['required', {
                    list_of: [{
                        nested_object: {
                            anchor_uid: ['strict_string', 'not_empty', 'required'],
                            sfd: ['strict_integer', { one_of: [250, 500, 1000] }, 'required'],
                            auto_sync: ['strict_bool', 'required'],
                        }
                    }]
                }],
                distributors: ['required', {
                    list_of: [{
                        nested_object: {
                            anchor_uid: ['strict_string', 'not_empty', 'required'],
                            master_uid: ['strict_string', 'not_empty_allow_null', 'required_allow_null'],
                            slot: ['strict_integer', 'required', { min_number: 0 }, { max_number: 254 }],
                        }
                    }]
                }]
            }
        }, 'required' ]
    };

    async execute(params) {
        const networkSetting = await new NetworkSetting().save(params);

        await this.notificator.notify(NetworkSetting.savedMessage(networkSetting));

        return networkSetting;
    }
}
