import ModelBase from '../ModelBase.mjs';
import { toISOTime } from '../../utils/index.mjs'

export class Association extends ModelBase {
    /**
     *
     * @param association {SolutionTrackableAssociation}
     * @return {PlatformAssociation}
     */
    static formatTrackableAssociation(association) {
        return {
            trackable_uid: association.trob_uid,
            mac: association.mac_address && association.mac_address.toUpperCase(),
            created_at: toISOTime(association.created_at),
        }
    }

    /**
     *
     * @param association {SolutionWinAssociation}
     * @return {PlatformAssociation}
     */
    static formatAnchorAssociation(association) {
        return {
            anchor_uid: association.win_uid,
            mac: association.mac_address && association.mac_address.toUpperCase(),
            created_at: toISOTime(association.created_at),
        }
    }

    /**
     *
     * @param association {PlatformAssociation}
     * @return {PlatformAssociation}
     */
    static formatPlatformAssociation(association) {
        const { anchor_uid, trackable_uid } = association;
        const created_at = association.created_at.split('.')[0]+'.000Z';

        if (anchor_uid) {
            return {
                anchor_uid,
                mac: association.mac,
                created_at
            }
        }
        if (trackable_uid) {
            return {
                trackable_uid,
                mac: association.mac,
                created_at
            }
        }
        return null;
    }

    /**
     *
     * @return {Promise<SolutionTrackableAssociation|SolutionWinAssociation[]>}
     */
    async collectSolutionEntities() {
        const trackableAssociations = (await this.solutionRepositoryInstance.association.readAllTkoAssoc()).map(Association.formatTrackableAssociation);
        const winAssociations = (await this.solutionRepositoryInstance.association.readAllWinAssoc()).map(Association.formatAnchorAssociation);
        return [ ...trackableAssociations, ...winAssociations ];
    }

    /**
     *
     * @return {Promise<PlatformAssociation[]>}
     */
    async collectPlatformEntities() {
        return (await this.platformRepositoryInstance.association.findAll()).map(Association.formatPlatformAssociation);
    }

    /**
     *
     * @param items {PlatformAssociation[]}
     * @return {Promise<void>}
     */
    async addToPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.association.save(item);
        }
    }

    /**
     *
     * @param {PlatformAssociation[]} items
     * @return {Promise<void>}
     */
    async removeInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.association.del(item);
        }
    }

    /**
     *
     * @param items {PlatformAssociation[]}
     * @return {Promise<void>}
     */
    async updateInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.association.save(item);
        }
    }
}
