import ModelBase from '../ModelBase.mjs';
import { toISOTime } from '../../utils/index.mjs';
import { NetworkId } from '../id/index.mjs';

export class Location extends ModelBase {
    /**
     *
     * @param location {SolutionLocation}
     * @returns {PlatformLocation}
     */
    static formatSolutionLocation(location) {
        return {
            uid: location.uid,
            name: location.name,
            created_at: toISOTime(location.created_at),
            updated_at: toISOTime(location.updated_at),
        }
    }
    
    /**
     *
     * @param location {PlatformLocation}
     * @return {PlatformLocation}
     */
    static formatPlatformLocation(location) {
        return {
            uid: location.uid,
            name: location.name,
            created_at: location.created_at,
            updated_at: location.updated_at,
        }
    }

    /**
     *
     * @return {Promise<PlatformLocation[]>}
     */
    async collectSolutionEntities() {
        return (await this.solutionRepositoryInstance.location.readAll()).map(Location.formatSolutionLocation)
    }

    /**
     *
     * @return {Promise<PlatformLocation[]>}
     */
    async collectPlatformEntities() {
        return (await this.platformRepositoryInstance.location.findAll()).map(Location.formatPlatformLocation)
    }
    
    /**
     *
     * @param items {PlatformLocation[]}
     * @return {Promise<void>}
     */
    async addToPlatform(items) {
        for (const item of items) {
            item.network_id = Number(await this.platformRepositoryInstance.networkId.nextValue());
            await this.platformRepositoryInstance.location.save(item);
        }
    }
    
    /**
     *
     * @param items {PlatformLocation[]}
     * @returns {Promise<void>}
     */
    async removeInPlatform(items) {
        for (const item of items) {
            const location = await this.platformRepositoryInstance.location.findByUid(item.uid);
            if (!location) {
                return;
            }
            await this.platformRepositoryInstance.location.deleteByUid(location.uid);
            await new NetworkId().remove(location.network_id);
        }
    }
    
    /**
     *
     * @param items {PlatformLocation[]}
     * @returns {Promise<void>}
     */
    async updateInPlatform(items) {
        for (const item of items) {
            await this.platformRepositoryInstance.location.save(item);
        }
    }
}
