import RepoBase from '../RepoBase.mjs';

/**
 * @typedef SolutionTrackableAssociation {Object}
 * @property trob_uid {String}
 * @property mac_address {String}
 * @property created_at {Data}
 */

/**
 * @typedef SolutionWinAssociation {Object}
 * @property win_uid {String}
 * @property mac_address {String}
 * @property created_at {Data}
 */

/**
 * @class AssocRepository
 * @description create access for the trackable and anchor associations in the solutions DB
 */
export class AssocRepository extends RepoBase {
    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     */
    constructor(options) {
        const { repositoryInstance } = options;
        super(repositoryInstance);
    }

    /**
     * @method
     * @return {Promise<SolutionTrackableAssociation[]>} returns result of search by key
     */
    async readAllTkoAssoc() {
        return (await this.repoInstance.queryAsync(AssocRepository.readAllTkoAssocSql) || []);
    }

    /**
     * @method
     * @return {Promise<SolutionWinAssociation[]>} returns result of search by key
     */
    async readAllWinAssoc() {
        return (await this.repoInstance.queryAsync(AssocRepository.readAllWinAssocSql) || []);
    }
    
    static get readAllTkoAssocSql() {
        return `select
                    trob_uid, mac_address, created_at
                from view_trackable_objects_associations
                where removed_at > NOW();`
    }
    
    static get readAllWinAssocSql() {
        return `select
                    win_uid, mac_address, created_at
                from view_winanchor_associations
                where removed_at > NOW();`
    }
}
