import RepoBase from '../RepoBase.mjs';

/**
 * @typedef SolutionSublocation {Object}
 * @property uid {String}
 * @property name {String}
 * @property sublocation_id {Number}
 * @property created_at {Date}
 * @property updated_at {Date}
 * @property loc_uid {String}
 * @property offset_z {Number}
 */
/**
 * @class SublocationRepository
 * @description create access for the sublocations in the solutions DB
 */
export class SublocationRepository extends RepoBase {
    /**
     * @param {Object} options - repository options
     * @param {Object} options.repositoryInstance - database connection
     */
    constructor(options) {
        const { repositoryInstance } = options;
        super(repositoryInstance);
    }

    /**
     * @method
     * @return {Promise<SolutionSublocation[]>} returns result of search by key
     */
    async readAll() {
        return (await this.repoInstance.queryAsync(SublocationRepository.readAllSql) || []);
    }

    /**
     * @method
     * @param {number} id - sublocation numeric identifier
     * @param {string} uid - sublocation unique identifier
     * @return {Promise<void>} returns result of search by key
     */
    async update(id, uid) {
        this.repoInstance.queryAsync('UPDATE sublocations SET `sequence` = ? where uid = ?;', [id, uid]);
    }
    
    static get readAllSql() {
        return `select
                    uid, name, created_at, updated_at, loc_uid, offset_z
                from view_sublocations;`
    }
}
