import sinon from 'sinon';
import assert from 'assert';
import { SolutionDBPoolFactory } from '../lib/db/solutionDBClient.js';
import { Service, fakeQueryAsync } from './helpers.mjs';
import dbData from './solution-db-data.json' assert { type: 'json' };

describe('Associations transfer', function () {
    let solutionPoolStub;
    let platformRepository;
    let resultAssociations;

    before(async function () {
        solutionPoolStub = sinon.stub(SolutionDBPoolFactory, 'createSolutionDBPool');
        solutionPoolStub.callsFake(() => ({ queryAsync: fakeQueryAsync }));
        const result = await Service().init();
        platformRepository = result.platformRepository;
    });
    after(async function () {
        SolutionDBPoolFactory.createSolutionDBPool.restore();

        if (Array.isArray(resultAssociations)) {
            for (const association of resultAssociations) {
                await platformRepository.association.del(association);
            }
        }
    });

    it('Move associations', async function () {
        resultAssociations = await platformRepository.association.findAll();

        let association = resultAssociations.find((assoc) => assoc.mac === 'E4956EACE53A');
        assert.deepStrictEqual(association, dbData.associations.expect.association1);
        association = resultAssociations.find((assoc) => assoc.mac === 'E4956EA30064');
        assert.deepStrictEqual(association, dbData.associations.expect.association2);
    });
});
