import sinon from 'sinon';
import assert from 'assert';
import { SolutionDBPoolFactory } from '../lib/db/solutionDBClient.js';
import dbData from './solution-db-data.json' assert { type: 'json' };
import { fakeQueryAsync, Service } from './helpers.mjs';

describe('Locations transfer', function() {
    let solutionPoolStub;
    let platformRepository;
    let resultLocations;
    
    before(async function() {
        solutionPoolStub = sinon.stub(SolutionDBPoolFactory, 'createSolutionDBPool');
        solutionPoolStub.callsFake(() => ({ queryAsync: fakeQueryAsync }));
        const result = await Service().init();
        platformRepository = result.platformRepository;
    });
    after(async function() {
        SolutionDBPoolFactory.createSolutionDBPool.restore();
        
        if (Array.isArray(resultLocations)) {
            for (const location of resultLocations) {
                await platformRepository.location.del({ uid: location.uid });
            }
        }
    });
    
    it('Move locations', async function() {
        resultLocations = await platformRepository.location.findAll();
        let location = resultLocations.find((loc) => loc.name === 'UA dev office');
        dbData.locations.expect.location1.network_id = location.network_id;
        assert.deepStrictEqual(location, dbData.locations.expect.location1);
        location = resultLocations.find((loc) => loc.name === 'IllusionTestLocation');
        dbData.locations.expect.location2.network_id = location.network_id;
        assert.deepStrictEqual(location, dbData.locations.expect.location2);
    });
});
