import { RepoError } from './models/index.mjs';

export default class RepoBase {
    /**
     * @typedef {Object} RepoBase
     */

    /**
     * @param {Object} repoInstance - DB connection Pool
     * @param {String} document - DB table name
     */
    constructor(repoInstance, document) {
        this.repoInstance = repoInstance;
        this.document = document;
    }

    /**
     * @method
     * @return {Promise<Array<Object>>} returns entities list
     */
    async findAll() {
        return this.repoInstance.queryAsync(`SELECT * FROM ${this.document}`).catch((err) => {
            throw new RepoError(err);
        });
    }

    /**
     * @method
     * @return {Promise<Array<Object>>} returns entities list
     */
    async findByUid(uid) {
        return this.repoInstance.queryAsync(`SELECT * FROM ${this.document} WHERE uid = ?`, [uid]).catch((err) => {
            throw new RepoError(err);
        });
    }

    /**
     * @method
     * @return {Promise<void>}
     */
    async save(params) {
        await this.repoInstance.queryAsync(`INSERT INTO ${this.document} SET ?`, [params]).catch((err) => {
            throw new RepoError(err);
        });
    }

    /**
     * @method
     * @return {Promise<void>}
     */
    async updateById(params, uid) {
        await this.repoInstance
            .queryAsync(`UPDATE ${this.document} SET ? WHERE uid = ?`, [params, uid])
            .catch((err) => {
                throw new RepoError(err);
            });
    }

    /**
     * @method
     * @return {Promise<void>}
     */
    async delete(uid) {
        await this.repoInstance.queryAsync(`DELETE FROM ${this.document} WHERE uid = ?`, [uid]).catch((err) => {
            throw new RepoError(err);
        });
    }
}
